<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\Manager\Mongo\Manager;
use Teknisa\Libs\Exception\CustomException;
use Teknisa\Libs\Util\InstanceProvider;
use \Util\OpensslCrypt;
use Util\ApiDataset;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Response;

class Api {

    protected $widgetService;
	protected $widgetDataService;
    /**  @var \Service\Api */
	protected $apiService;
	protected $mongoManager;
	protected $dataSourceName = 'structure';
	const clientId = "zmart5178153by60953518945f9hakt19a50a4";
	const clientSecret = "zmart49996c21f780fe30badd6521931a5sac9";

	public function __construct()

	{
        $this->widgetService = InstanceManager::getInstance()->getService('\Service\Widget');
        $this->widgetDataService = InstanceManager::getInstance()->getService('\Service\WidgetData');
        $this->apiService = InstanceManager::getInstance()->getService('\Service\Api');
        $this->mongoManager = InstanceManager::getInstance()->getService('organizationMongoManager');
    }

	public function findBiggerStructure(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
        	$conditions = $request->getFilterCriteria()->getConditions();
            $environment = Utilities::getValueInFilterDataSourceByName($conditions, 'environment', true);
			$zmartStructureFilterId = Utilities::getValueInFilterDataSourceByName($conditions, 'zmartStructureFilterId', true);
			$this->apiService->setSessionVariables($environment);
			$structure = $this->apiService->findBiggerStructure($zmartStructureFilterId);
			$response->addDataSet(new DataSet('findBiggerStructure', $structure));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

	public function findbuildWidgetByViewId(DTO\Request\Row $request, DTO\Response $response)
	{
		try {
            $row = $request->getRow();
			$environment = $row['environment'];
			$dataRow = $row['dataRow'];

			$this->apiService->setSessionVariables($environment);
			$widget = $this->buildWidgetByViewId($dataRow);
			$response->addDataSet(new DataSet('findbuildWidgetByViewId', $widget));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

	private function buildWidgetByViewId($row)
	{
        if (!empty($row['viewId'])) {
            $viewId      = $row['viewId'];
            $facts       = isset($row['facts'])                         ? $row['facts']             : array();
            $orderBy     = isset($row['orderBy'])                       ? $row['orderBy']           : array();
            $period      = isset($row['period'])                        ? $row['period']            : array();
            $structureId = isset($row['structureId'])                   ? $row['structureId']       : null;
            $structureName = isset($row['structureName'])               ? $row['structureName']       : null;
            $dimension   = isset($row['dimension'])                     ? $row['dimension']         : null;
            $limitRows   = isset($row['limitRows'])                     ? $row['limitRows']         : null;
            $drillDownFilters = isset($row['drillDownFilters'])         ? $row['drillDownFilters']  : null;
            $valuePrecision = isset($row['valuePrecision'])             ? $row['valuePrecision']    : null;
            $serieColors = isset($row['serieColors']) && $row['serieColors']  ? $row['serieColors'] : null;
            $dimensionX     = isset($row['dimensionX'])      ? $row['dimensionX']       : null;
            $dimensionY     = isset($row['dimensionY'])      ? $row['dimensionY']       : array();
            $widget = $this->widgetService->buildWidgetToRender($viewId, $structureId,
                $period, $dimension, $facts, $orderBy, $limitRows, $drillDownFilters,
                $valuePrecision, null, null, $serieColors, null, $dimensionX, $dimensionY, $structureName);

            if(isset($widget['legendClosed'])){
                $chartLegendClosedValue  = $widget['legendClosed'];
                unset($widget['legendClosed']);
                $widget['chartLegendClosed']  = $chartLegendClosedValue;
            }
            $this->widgetService->getWidgetLastImportDate($widget);
            return array($widget);
        } else {
        	throw new Exception("INVALID_PARAMETERS", 500);
        }
	}

	public function fetchWidgetData(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $environment = Utilities::getValueInFilterDataSourceByName($conditions, 'environment');
            $row = Utilities::getValueInFilterDataSourceByName($conditions, 'value');
            $this->apiService->setSessionVariables($environment);
            $data = $this->doFetchWidgetData($row);
			$response->addDataSet(new DataSet('fetchWidgetData', $data));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

	 public function doFetchWidgetData($row)
	 {
        $data = array();
        if(!empty($row['viewId']) && (!empty($row['structureId']) || !empty($row['structureTypeId']))) {
            $viewId             = $row['viewId'];
            $structureId        = isset($row['structureId'])         ? $row['structureId']          : null;
            $structureTypeId    = isset($row['structureTypeId'])     ? $row['structureTypeId']      : null;
            $period             = isset($row['period'])              ? $row['period']               : null;
            $dimension          = isset($row['dimension'])           ? $row['dimension']            : null;
            $limitRows          = isset($row['limitRows'])           ? $row['limitRows']            : null;
            $facts              = isset($row['facts'])               ? $row['facts']                : array();
            $orderBy            = isset($row['orderBy'])             ? $row['orderBy']              : array();
            $drillDownFilters   = isset($row['drillDownFilters'])    ? $row['drillDownFilters']     : array();
            $groupByStructure   = isset($row['groupByStructure'])    ? $row['groupByStructure']     : null;
            $previousCondition  = isset($row['previousConditions'])  ? $row['previousConditions']   : array();
            $postConditions     = isset($row['postConditions'])      ? $row['postConditions']       : array();
            $dimensionX         = isset($row['dimensionX'])          ? $row['dimensionX']           : null;
            $dimensionY         = isset($row['dimensionY'])          ? $row['dimensionY']           : array();
            $filterOrderByTotal = isset($row['orderByTotal'])        ? $row['orderByTotal']         : null;
            $structureId = is_array($structureId) ? $structureId : array($structureId);
            if ($period && isset($period['period'])){
                $period = $period['period'];
            }
            $data = $this->widgetDataService->fetchWidgetData($viewId, $structureId, $structureTypeId, $period,
                $facts, $dimension, $orderBy, null, $limitRows, $drillDownFilters, $groupByStructure,
                $previousCondition, $postConditions, $filterOrderByTotal, $row,$dimensionX,$dimensionY);

        }

    	return $data;
    }

    public function apiDataSource(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $conditions = $request->getFilterCriteria();
            $data = $this->apiService->findApiDataSource($conditions);
            $response->addDataSet($data);
        }catch (\Exception $e){
            throw  $e;
        }
    }

    public function saveTokenTemp(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();

            $data = $this->apiService->saveTokenTemp($row);

            $response->addDataSet(new DataSet(\Service\Api::API_DATA_SOURCE_COLLECTION, $data));
        }catch (\Exception $e){
            throw  $e;
        }
    }

    public function saveApiDataSource(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            /** @var \Zeedhi\Framework\DTO\Row $row */
            $row = $request->getDataSet()->getRows()[0];
            $this->apiService->validateApiPeriodLength($row["FIELD_DATE_PERIOD"]);
            $data = $this->apiService->saveApiDataSource($row);

            $response->addDataSet(new DataSet(\Service\Api::API_DATA_SOURCE_COLLECTION, $data));
        }catch (CustomException $e){
            $error = \Teknisa\Libs\Util\Utilities::getErrorObjectFromException( $e);
            $response->setCriticalError($error);
        }catch (\Exception $e){
            $response->setCriticalError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(),$e->getCode()) );
        }
    }

    public function deleteApiDataSource(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $data = $this->apiService->deleteApiDataSource($rows);
            $response->addDataSet(new DataSet(\Service\Api::API_DATA_SOURCE_COLLECTION, $data));
        }catch (\Exception $e){
            throw  $e;
        }
    }

}
