<?php

namespace Controller;


class Exception extends \Exception{

    const RESPONSE_CONNECTION_NOT_FOUND = "Conexão não encontrada: ";
    const RESPONSE_FAIL_AT_CREATE = "Não foi possível criar a conexão: ";
    const BASE_RESPONSE_CONNECTION_FAIL = "Não foi possível conectar com o banco de dados.";
    const RESPONSE_CONNECTION_FAIL = "Não foi possível conectar com o banco de dados, as colunas antigas foram mantidas.";
    const BASE_RESPONSE_EXECUTE_QUERY_FAIL = "Falha na execução da query. ";
    const RESPONSE_EXECUTE_QUERY_FAIL = "Não foi possível sincronizar as colunas. Falha na execução da query: ";
    const RESPONSE_SYNC_JOIN_DATASOURCE_COLUMNS = "Não foi possível sincronizar as colunas. As colunas da base original não foram encontradas. ";
    const CODE_FAIL_CONNECTION = 404;
    const CODE_QUERY_EXECUTION_FAIL = 500;

    public static function buildErrorJSON(string $message, array $params = []){
        $error = array();
        $error["message"] = $message;
        if(!empty($params)){
            $error["params"] = $params;
        }
        return json_encode($error);
    }

    public static function connectionNotFound($connectionName = "") {
        return new static(self::RESPONSE_CONNECTION_NOT_FOUND.$connectionName, 1);
    }

    public static function createConnectionError($connectionName) {
        return new static(self::RESPONSE_FAIL_AT_CREATE.$connectionName, 1);
    }

    public static function baseConnectFail() {
        return new static(self::BASE_RESPONSE_CONNECTION_FAIL, self::CODE_FAIL_CONNECTION);
    }

    public static function connectFail() {
        return new static(self::RESPONSE_CONNECTION_FAIL, self::CODE_FAIL_CONNECTION);
    }

    public static function baseExecuteQueryError($query, \Exception $previousException) {
        return new static(self::BASE_RESPONSE_EXECUTE_QUERY_FAIL."Error: {$previousException->getMessage()}. The executed query was: \n {$query}", self::CODE_QUERY_EXECUTION_FAIL);
    }
    
    public static function executeQueryError($query, \Exception $previousException) {
        return new static(self::RESPONSE_EXECUTE_QUERY_FAIL."{$query}.\n Error: {$previousException->getMessage()}", self::CODE_QUERY_EXECUTION_FAIL);
    }
    
    public static function syncJoinDataSourceColumnsError($originalDataSourceName) {
        return new static(self::RESPONSE_SYNC_JOIN_DATASOURCE_COLUMNS."{$originalDataSourceName}");
    }
    
    public static function throwNewException($errorMessage){
        return new static($errorMessage);
    }
} 