<?php


namespace Controller;


use Util\Utilities;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Organization
{
    public function organizationInactive(){
        ini_set("memory_limit", "1024M");
        ini_set('max_execution_time', '360000');
        set_time_limit(360000);
        /** @var \Service\Organization $organizationService */
        $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
        echo "Starting Inactive Organization!\n";
        $start = microtime(true);
        try{
            $organizationService->removeImportationOrganization();
            $organizationService->setAllOrganization();
            $organizationService->removeImportationWithoutDataSource();
        }catch (\Exception $e){
            echo $e->getMessage();
            throw $e;
        }
        $end = microtime(true);
        $total = $end - $start;
        echo("Done Inactive Organization!\n Tempo = $total\n");
    }

    public function findAll(\Zeedhi\Framework\DTO\Request\Filter $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $isPrimary =  \Util\Utilities::getValueInFilterDataSourceByName($filters, 'IS_PRIMARY', true);

            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizations = $organizationService->findAll($filters, $page, $pageSize, $orderBy, null, $isPrimary);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('allOrganizations', $organizations));
        } catch (\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findClientsMongo(\Zeedhi\Framework\DTO\Request\Filter $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $room =  \Util\Utilities::getValueInFilterDataSourceByName($filters, 'ROOM', true);

            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizations = $organizationService->findClientsMongo($filters, $room);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('allOrganizations', $organizations));
        } catch (\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function roomBalancing(\Zeedhi\Framework\DTO\Request\Filter $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();

            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizations = $organizationService->roomBalancing();
            Utilities::setDataSetDefault($request,$response,$organizations);
        } catch (\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function clientsMongoSave(\Zeedhi\Framework\DTO\Request\DataSet $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $rows = $request->getDataSet()->getRows();
            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizations = $organizationService->clientsMongoSave($rows[0]);
            Utilities::setDataSetDefault($request,$response,$organizations);
        } catch (\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function execRoomBalancing(\Zeedhi\Framework\DTO\Request $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $params = $request->getParameters();
            $notRejeration = empty($params['NOT_REJERATION']) ? false : $params['NOT_REJERATION'];
            /** @var \Service\Organization $organizationService */
            $organizationService = InstanceManager::getInstance()->getService('\Service\Organization');
            $organizationService->execRoomBalancing($notRejeration);
        } catch (\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

}
