<?php

namespace Service;


use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class DefaultRelations {

    protected $dataSourceName = "defaultRelations";

    /** @var Manager*/
    protected $mongoManager;

    public function __construct(Manager $mongoManager) {
        $this->mongoManager = $mongoManager;
    }

    public function dataSetSave(DataSet $dataSet) {
        $dataSet->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->persist($dataSet);
    }

    public function filterFind(FilterCriteria $filterCriteria) {
        $filterCriteria->setDataSourceName($this->dataSourceName);
        return $this->mongoManager->findBy($filterCriteria);
    }

    public function findByStructureType($structureTypeId) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('structureType', FilterCriteria::EQ, $structureTypeId);
        return $this->filterFind($filterCriteria);
    }

    public function findByStructureTypes(array $structureTypeIds) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('structureType', FilterCriteria::IN, $structureTypeIds);
        return $this->filterFind($filterCriteria);
    }

    public function insertNewDefault(array $relation) {
        $dataSet = $this->findByStructureType($relation['structureType']);
        $oldRelations = $dataSet->getRows();
        if(empty($oldRelations)) {
            return $this->dataSetSave(new DataSet($this->dataSourceName, array($relation)));
        }
    }

    public function updateOrInsertDefaultRelation(array $relation) {
        $relationWithId = $this->setDefaultRelationIdIfExists($relation);
        return $this->dataSetSave(new DataSet($this->dataSourceName, array($relationWithId)));
    }

    /**
     * @param $relation
     * @return $relation with _id if exists a default relation with the same name
     */
    protected function setDefaultRelationIdIfExists($relation) {
        $dataSet = $this->findByStructureType($relation['structureType']);
        $oldRelations = $dataSet->getRows();
        if(!empty($oldRelations)) {
            $oldRelation = $oldRelations[0];
            $relation['_id'] = $oldRelation['_id'];
        }
        return $relation;
    }
} 