<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Service\Manager\Mongo\Manager;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;

class Organization {

    /**
     * @var Manager
     */
	protected $mongoManager;
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Operador
     */
    private $operador;
    private $allDataSource;

    protected static $collectionMeasures = "yesterdayImportMeasures";
    protected static $roomLoadBalance = "roomLoadBalance";

    public function __construct(Manager $organizationMongoManager,EntityManager $entityManager, Operador $operador)
	{
        $this->mongoManager = $organizationMongoManager;
        $this->entityManager = $entityManager;
        $this->operador = $operador;
    }

    public function hasDataSource($importSchedule)
    {
        $validate = true;
        $nrorg = $importSchedule['organizationId'];
        $dataSourceId = $importSchedule['dataSource'];
        if (!empty($nrorg) && empty($this->allDataSource[$nrorg])) {
            $mongoManagerService = LogImportation::buildManager($nrorg);
            $filterCriteria = new ZmartFilterCriteria('dataSource', array());
            $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
            $filterCriteria->setPage(1);
            $data = $mongoManagerService->simpleFindPagination($filterCriteria);
            $this->allDataSource[$nrorg] = array();
            foreach ($data as $dataSource) {
                $row = array();
                $row['query'] = $dataSource['query'];
                $row['name'] = $dataSource['name'];
                $id = $dataSource['_id']->__toString();
                $this->allDataSource[$nrorg][$id] = $row;
            }
        }
        if (empty($this->allDataSource[$nrorg][$dataSourceId])) {
            $validate = false;
        }
        return $validate;
    }
    public function removeImportationWithoutDataSource()
    {

        $mongoManagerZmart = LogImportation::buildManager();
        $filterCriteria = new ZmartFilterCriteria('importSchedule');
        $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
        $filterCriteria->setPage(1);
        $dataSet = $mongoManagerZmart->simpleFindPagination($filterCriteria);
        $delete = array();
        foreach ($dataSet as $row) {
            if(!empty($row['dataSource']) && !empty($row['organizationId'])){
                $hasDataSource = $this->hasDataSource($row);
                if(!$hasDataSource){
                    $delete[] = $row;
                }
            }
        }
        $dataSetDelete = new DataSet('importSchedule', $delete);

        echo("Foram removidas ".count($delete). " importações por não ter fonte de dados.\n ");
        $mongoManagerZmart->delete($dataSetDelete);
    }

    public function removeImportationOrganization()
    {
        $idOrgActive = $this->operador->findZmartOrg();
        if(empty($idOrgActive)){
            throw  new \Exception("NO_ZMART_CLIENT_FOUND");
        }

        $idOrgActive = explode(',',$idOrgActive[0]['NRORG']);
        $filterCriteria = new FilterCriteria("importSchedule", array(
                array(
                    "columnName" => "organizationId",
                    "value" => $idOrgActive,
                    "operator" => "NOT_IN")
            )
        );
        $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
        $filterCriteria->setPage(1);
        $dataset = $this->mongoManager->findBy($filterCriteria);
        echo("Foram removidas ".count($dataset->getRows()). " importações.\n ");
        $this->mongoManager->delete($dataset);
    }

    public function setAllOrganization($notEcho = false)
    {
        $idOrgActive = $this->operador->findZmartOrgToMongo();
        $dataset = new DataSet('ZmartOrg',$idOrgActive);
        $this->mongoManager->dropCollection('ZmartOrg');
        $this->mongoManager->persist($dataset);
        if(!$notEcho){
            echo("Hoje tem um total de  ".count($dataset->getRows()). " clientes do Zmart.\n ");
        }
    }

    public function findAllOrganizacoes($conditions, $limit, $offset)
    {
        $query = <<<SQL
        SELECT *
          FROM ( SELECT O.NRORG, O.NMRAZSOCIALORG, O.NMORGANIZACAO, O.PREPSTATUS, O.IDATIVO,
                        O.IDTIPOINSCRORG, O.NRORGPRINCIPAL, O.PREPSTATUS AS ORGSTATUS,
                        DECODE(O.NRORGPRINCIPAL, NULL, 'P', 'S') AS IS_PRIMARY,
                        DECODE(O.NRORGPRINCIPAL, NULL, O.IDORGSAAS, OM.IDORGSAAS) AS IDORGSAAS
                   FROM ORGANIZACAO O
              LEFT JOIN ORGANIZACAO OM ON (O.NRORGPRINCIPAL = OM.NRORG))
         WHERE 1 = 1
              $conditions
SQL;

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $entityManager = \Util\InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true);
        $entityManager = $entityManager ? $entityManager : \Util\InstanceProvider::getEntityManager();
        $result = $entityManager->getConnection()->fetchAll($query);
        return ($result) ? $result : [];
    }

    public function findAll($filters, $page, $pageSize, $orderBy, $isIntId = true, $isPrimary = null) {
        $conditions = Utilities::createQueryFiltered($filters);
        if($isPrimary && $isPrimary == "S"){
            $conditions .= "AND NRORGPRINCIPAL IS NOT NULL";
        } else if($isPrimary && $isPrimary == "P"){
            $conditions .= "AND NRORGPRINCIPAL IS NULL";
        }
        foreach($orderBy as $idx => $order) {
            $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $organizacoes = $this->findAllOrganizacoes($conditions, $limit, $offset);
        if($isIntId) {
            foreach ($organizacoes as &$organizacao) {
                $organizacao['NRORG'] = intval($organizacao['NRORG']);
            }
        }
        return $organizacoes;
    }

    public function findClientsMongo($filters, $room = null){
        $orgs = $this->findZmartOrgs($filters);
        $organizationId = Utilities::getArrayFromAssociativeArray($orgs,'NRORG');
        $filterNrorg = array(
            "columnName"=>"organizationId",
            "value" => $organizationId,
            "operator" => FilterCriteria::IN
        );
        $filterMongo = array($filterNrorg);
        if(!empty($room)){
            $filterZmart = array(
                "columnName"=>"clientRoom",
                "value" => $room,
                "operator" => FilterCriteria::IN
            );
            $filterMongo = array($filterNrorg,$filterZmart);
        }
        $filterCriteria = new FilterCriteria('connection', $filterMongo, null, 100000);
        $connections =  $this->mongoManager->findBy($filterCriteria)->getRows();
        $connections = Utilities::indexingArrayAssociative($connections,"organizationId");
        foreach ($orgs as &$org) {
            $room = null;
            if(!empty($connections[$org['NRORG']]['clientRoom'])){
               $room = $connections[$org['NRORG']]['clientRoom'];
            }
            $org['ROOM'] = $room;
        }

        return $orgs;
    }
    public function clientsMongoSave($row){
        $filterNrorg = array(
            "columnName"=>"organizationId",
            "value" => $row['NRORG'],
            "operator" => FilterCriteria::EQ
        );
        $filterMongo = array($filterNrorg);
        $filterCriteria = new FilterCriteria('connection', $filterMongo, null, 100000);
        $dataBefore =  $this->mongoManager->findBy($filterCriteria)->getRows();
        $connections =  $this->mongoManager->findBy($filterCriteria)->getRows()[0];
        $connections['clientRoom'] = $row['ROOM'];
        if(empty($connections['clientRoom'])){
            $connections['clientRoom'] = null;
        }
        $connections = $this->mongoManager->persist(new DataSet('connection', array($connections)));
        $dataAfter =  $this->mongoManager->findBy($filterCriteria)->getRows();
        Utilities::setLog($dataBefore, $dataAfter, 'connection', $row['NRORG'], '',$this->mongoManager);
        return $connections;
    }

    public function roomBalancing()
    {
        $rooms = Utilities::getroomLoadBalance();
        $dataset = [];
        foreach ($rooms as $room) {
            $dataset[] = array(
                "ID" => $room,
                "NAME" => $room
            );
        }
        return $dataset;
    }

    public function findZmartOrgs($filters) {
        $conditions = Utilities::createQueryFiltered($filters);
        $sql = <<<SQL
SELECT * FROM (
                 SELECT ORG.NRORG,ORG.NMRAZSOCIALORG,ORG.NMORGANIZACAO,ORG.IDORGSAAS
                         FROM OM_PRODUTO OPR
                         JOIN OM_PRODUTO_ORG OPO ON OPO.NRPRODUTO_ID = OPR.ID
                         JOIN ORGANIZACAO ORG ON (ORG.NRORG = OPO.NRORG_ID)
                        WHERE OPR.NREXTERNAL_PRODUCT_ID = :ZMART_ID
                          AND ORG.IDATIVO != 'N')
                          WHERE 1 = 1 
                          $conditions
SQL;

        $param = array(
            'ZMART_ID' => Utilities::getProjectId()
        );
        return \Util\InstanceProvider::getConnectionCustom()->updateVpdConnection(20,true)->getConnection()->fetchAllAssociative($sql,$param);

    }

    public function execRoomBalancing($notRejeration = false){
        if(!$notRejeration){
            $measurementsArray = $this->getMessaurementsYesterday();
        } else {
            $filterCriteria = new ZmartFilterCriteria('yesterdayImportMeasures', array());
            $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
            $filterCriteria->setPage(1);
            $filterCriteria->addOrderBy("sizeDay",FilterCriteria::ORDER_DESC);
            $measurementsArray = $this->mongoManager->findBy($filterCriteria)->getRows();
        }

        if(empty($measurementsArray)){
            throw new \Exception("Sem medidas para do dia anterior");
        }

        $roomLoadBalanceIndexed = $this->createRoomLoadBalanceIndexed($measurementsArray);

        foreach ($roomLoadBalanceIndexed as $room => $loadBalance) {
            $criteria =  [
                [
                    "columnName" => "organizationId",
                    "value" => $loadBalance['NRORG'],
                    "operator" => FilterCriteria::IN
                ]
            ];
            $filterCriteria = new ZmartFilterCriteria('connection', $criteria);
            $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
            $filterCriteria->setPage(1);
            $manager = Utilities::buildManager();
            $row = ["clientRoom" => strval($room)];
            $updateMongo = array('$set' => $row);
            $manager->update('connection', $filterCriteria, $updateMongo);

        }


    }

    /**
     * @return array
     */
    public function getOrganizationsRoomBalancing(): array
    {
        $filter = array(
            array(
                "columnName" => "IDORGSAAS",
                "value" => ['S', 'H'],
                "operator" => FilterCriteria::IN
            ),
            array(
                "columnName" => "NRORG",
                "value" => ['5', '20'],
                "operator" => FilterCriteria::NOT_IN
            )
        );
        $organisations = $this->findClientsMongo($filter);
        return $organisations;
    }

    /**
     * @param string $stringYesterday
     * @return ZmartFilterCriteria
     */
    public function getFilterImportHistory(string $stringYesterday): ZmartFilterCriteria
    {
        $filterCriteria = new ZmartFilterCriteria('importHistory', array(
            array(
                "columnName" => "createDate",
                "value" => array(
                    $condition['value'][0] = Utilities::convertStringDataBrToMongoDate($stringYesterday, ' 00:00:00'),
                    $condition['value'][1] = Utilities::convertStringDataBrToMongoDate($stringYesterday, ' 23:59:59'),
                ),
                "operator" => "BETWEEN"
            ),
            array(
                "columnName" => "importType",
                "value" => "scheduledBI",
                "operator" => "="
            )

        ));
        $filterCriteria->setPageSize(LogImportation::PAGE_SIZE);
        $filterCriteria->setPage(1);
        return $filterCriteria;
    }

    /**
     * @param $nowMongo
     * @param $nrorg
     * @return array
     */
    public function setInitMeasurements($nowMongo, $nrorg): array
    {
        $measurements = array();
        $measurements['createDate'] = $nowMongo;
        $measurements['CREATED'] = new \MongoDB\BSON\UTCDateTime();
        $measurements['NRORG'] = $nrorg;
        $measurements['sizeDay'] = 0;
        $measurements['elapsedTime'] = 0;
        return $measurements;
    }

    /**
     * @param array $measurements
     * @param $dataSourceId
     * @return array
     */
    public function setInitMessaurementsDatasource(array $measurements, $dataSourceId): array
    {
        if (empty($measurements[$dataSourceId])) {
            $measurements[$dataSourceId] = array();
            $measurements[$dataSourceId]['sizeDay'] = 0;
            $measurements[$dataSourceId]['elapsedTime'] = 0;
        }
        return $measurements;
    }

    /**
     * @param int $pageSize
     * @param int $imported
     * @param array $measurements
     * @param $dataSourceId
     * @param int $elapsedTime
     * @param $createDate
     * @return array
     */
    public function setsMeasurements(int $pageSize, int $imported, array $measurements, $dataSourceId, int $elapsedTime, $createDate): array
    {
        $measurements[$dataSourceId]['sizeDay'] += intval($pageSize) * intval($imported);
        $measurements[$dataSourceId]['elapsedTime'] += intval($elapsedTime);
        $measurements['sizeDay'] += intval($pageSize) * intval($imported);
        $measurements['elapsedTime'] += intval($elapsedTime);
        $measurements[$dataSourceId]['datesImportation'][] = $createDate;
        return $measurements;
    }

    /**
     * @param string $now
     * @param array $measurementsArray
     */
    public function insetMongoMeasurements(string $now, array $measurementsArray)
    {
        $this->mongoManager->dropCollection(self::$collectionMeasures . '_' . $now);
        $this->mongoManager->insertMany(self::$collectionMeasures . '_' . $now, $measurementsArray);
        $this->mongoManager->dropCollection(self::$collectionMeasures);
        $this->mongoManager->renameCollection(self::$collectionMeasures . '_' . $now, self::$collectionMeasures);
        $this->mongoManager->dropCollection(self::$collectionMeasures . '_' . $now);
    }

    /**
     * @return array
     * @throws \Exception
     */
    public function getMessaurementsYesterday(): array
    {
        $yesterday = new \DateTime();
        $today = $yesterday->format('Y-m-d');
        $now = $yesterday->format('YmdHis');
        $nowMongo = Utilities::convertStringDataTimeToMongoDate($today);
        $yesterday->modify('-1 day');
        $stringYesterday = $yesterday->format('d/m/Y');
        $organisations = $this->getOrganizationsRoomBalancing();
        $filterCriteria = $this->getFilterImportHistory($stringYesterday);
        $measurementsArray = [];
        foreach ($organisations as $organisation) {
            $nrorg = $organisation['NRORG'];
            $measurements = $this->setInitMeasurements($nowMongo, $nrorg);
            $manager = Utilities::buildManager($nrorg);
            $importHistories = $manager->simpleFindPagination($filterCriteria);
            foreach ($importHistories as $importHistory) {
                $dataSourceId = $importHistory['dataSourceId'];
                $measurements = $this->setInitMessaurementsDatasource($measurements, $dataSourceId);
                $pageSize = empty($importHistory['pageSize']) ? 0 : $importHistory['pageSize'];
                $imported = empty($importHistory['imported']) ? 0 : $importHistory['imported'];
                $elapsedTime = empty($importHistory['elapsedTime']) ? 0 : $importHistory['elapsedTime'];
                $createDate = empty($importHistory['createDate']) ? null : $importHistory['createDate'];
                if (is_numeric($pageSize)
                    && is_numeric($imported)
                    && $pageSize != 0
                    && $imported != 0
                ) {
                    $measurements = $this->setsMeasurements($pageSize, $imported, $measurements, $dataSourceId, $elapsedTime, $createDate);
                }
            }
            $measurementsArray[] = $measurements;
        }

        usort($measurementsArray, function ($a, $b) {
            return $a['sizeDay'] <=> $b['sizeDay'];
        });

        $this->insetMongoMeasurements($now, $measurementsArray);
        return $measurementsArray;
    }

    /**
     * @param array $measurementsArray
     * @return array
     */
    public function createRoomLoadBalanceIndexed(array $measurementsArray): array
    {
        $roomLoadBalance = Utilities::getroomLoadBalance();
        $roomLoadBalanceIndexed = array();
        foreach ($roomLoadBalance as $row) {
            $roomLoadBalanceIndexed[$row] = [
                "sizeDay" => 0,
                "elapsedTime" => 0,
                "NRORG" => [],
            ];
        }
        $freerRoom = $roomLoadBalance[0];
        foreach ($measurementsArray as $measurement) {
            $nrorg = $measurement['NRORG'];
            $sizeDay = $measurement['sizeDay'];
            $elapsedTime = $measurement['elapsedTime'];
            $roomLoadBalanceIndexed[$freerRoom]['NRORG'][] = $nrorg;
            $roomLoadBalanceIndexed[$freerRoom]['sizeDay'] += $sizeDay;
            $roomLoadBalanceIndexed[$freerRoom]['elapsedTime'] += $elapsedTime;
            foreach ($roomLoadBalanceIndexed as $room => $loadBalance) {
                if ($roomLoadBalanceIndexed[$room]['sizeDay'] < $roomLoadBalanceIndexed[$freerRoom]['sizeDay']) {
                    $freerRoom = $room;
                } else if ($roomLoadBalanceIndexed[$room]['sizeDay'] == $roomLoadBalanceIndexed[$freerRoom]['sizeDay']
                    && count($roomLoadBalanceIndexed[$room]['NRORG']) < count($roomLoadBalanceIndexed[$freerRoom]['NRORG'])) {
                    $freerRoom = $room;
                }
            }

        }

        $this->mongoManager->dropCollection(self::$roomLoadBalance);
        $this->mongoManager->insertMany(self::$roomLoadBalance, [$roomLoadBalanceIndexed]);
        return $roomLoadBalanceIndexed;
    }

}
