module.exports = function(grunt) {
    var config = 'assets/js/config/**/*.js';
    var zmartComponents = 'zmart_components/**/*.js';
    var utilPath = 'assets/js/util/**/*.js';
    var repositoryPath = 'assets/js/models/**/*.js';
    var factoryPath = 'assets/js/factories/**/*.js';
    var servicePath = [
        'assets/js/services/NotificationService.js',
        'assets/js/services/GoalsNotificationService.js',
        'assets/js/services/ChartService.js',
        'assets/js/services/AccessControlService.js',
        'assets/js/services/DataSourceService.js',
        'assets/js/services/FilterService.js',
        'assets/js/services/IndicatorFactory.js',
        'assets/js/services/StructureService.js',
        'assets/js/services/ViewPropertyFactory.js',
        'assets/js/services/ViewPropertyService.js',
        'assets/js/services/ViewTypeService.js',
        'assets/js/services/WidgetService.js',
        'assets/js/services/StyleService.js',
        'assets/js/services/ChartViewService.js',
        'assets/js/services/ViewService.js',
        'assets/js/services/DrillDownService.js',
        'assets/js/services/ExpressionInterpreter.js',
        'assets/js/services/GenericDashboardService.js',
        'assets/js/services/**/*.js'
    ];
    var controllerPath = [
        'assets/js/controllers/GoalsNotificationController.js',
        'assets/js/controllers/MainDashboardController.js',
        'assets/js/controllers/FilterController.js',
        'assets/js/controllers/GenericDashboardController.js',
        'assets/js/controllers/ChartViewController.js',
        'assets/js/controllers/ViewController.js',
        'assets/js/controllers/DashboardIndicatorController.js',
        'assets/js/controllers/GoalsController.js',
        'assets/js/controllers/IndicatorController.js',
        'assets/js/controllers/**/*.js'
    ];
    var libFiles = [
        'zh-grid-navigation/src/GridNavigationLibrary.js',
        'zh-customizations-service/dist/zhCustomizationsService.js',
        'zh-filter-preferences/dist/zhFilterPreferences.js',
        'zh-layout-preferences/dist/zhLayoutPreferences.js',
        'zh-customization-preferences/dist/zhCustomizationsPreferences.js',
        'zh-charts/dist/zh-charts.js',
        'zh-cache-bypass/src/CacheBypass.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-reports/dist/zhReports.js',
        'zh-crosstab/dist/zh-crosstab.min.js',
        'zh-rich-text/dist/zh-rich-text.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js',
        'zh-interval-component/dist/zh-interval-component.js'
    ];
    var specsPath = 'tests/unitTests/specs/**/*spec.js';
    var orderFiles = [];
    var helperPath = 'tests/unitTests/helpers/**/*.js';
    var compassPath = '';
    var jsonsPath = 'assets/json/containers/';

    grunt.initConfig({
        bump: {
		    options: {
		        files: ['bower.json', 'package.json'],
				pushTo: 'origin',
		        commitFiles: ["-a"],
		        push: true
		    }
		},
        mkdir: {
            all: {
                options: {
                    create: ['generated_reports']
                }
            }
        },
        compass: {
            dist: {
                options: {
                    force: true,
                    config: 'config.rb',
                    outputStyle: 'compressed'
                }
            }
        },
        concat: {
            none: {},
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'assets/dist/libs.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: 'assets/css/app.css'
            },
            zh: {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: ['libs/**/*.js', utilPath, repositoryPath, zmartComponents,factoryPath, servicePath, controllerPath, config],
                dest: 'assets/dist/zmart.js'
            },
            zh1: {
                src:  'bower_components/zh-select-autocomplete/dist/style/colors/styles-zeedhi.css',
                dest: 'bower_components/zh-select-autocomplete/dist/style/colors/styles-zeedhi.min.css'
            },
            zh2: {
                src:  'bower_components/zh-customizations-service/src/style/style.css',
                dest: 'bower_components/zh-customizations-service/src/style/style.min.css'
            },
            zh3: {
                src:  'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css',
                dest: 'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css'
            },
            zh4: {
                src:  'bower_components/zeedhi-frontend/assets/css/styles/styles-zeedhi.css',
                dest: 'bower_components/zeedhi-frontend/assets/css/styles/styles-zeedhi.min.css'
            },
            zh5: {
                src:  'bower_components/zh-reports/dist/style/style.css',
                dest: 'bower_components/zh-reports/dist/style/style.min.css'
            },
            zh6: {
                src:  'bower_components/zh-rich-text/dist/zh-rich-text.css',
                dest: 'bower_components/zh-rich-text/dist/zh-rich-text.min.css'
            },
            zh7: {
                src:  'assets/css/app.css',
                dest: 'assets/css/app.min.css'
            },
            zh8: {
                src:  'config/KpiConfig.js',
                dest: 'config/KpiConfig.min.js'
            },
            zh9: {
                src:  'assets/dist/libs.js',
                dest: 'assets/dist/libs.min.js'
            },
            zh10: {
                src:  'assets/js/environment.js',
                dest: 'assets/js/environment.min.js'
            },
            zh11: {
                src:  'config/Config.js',
                dest: 'config/Config.min.js'
            },
            zh12: {
                src:  'assets/js/engine.js',
                dest: 'assets/js/engine.min.js'
            },
            zh13: {
                src:  'assets/dist/zmart.js',
                dest: 'assets/dist/zmart.min.js'
            }
        },
        jshint: {
            all: ['Gruntfile.js', 'lib/**/*.js', config, specsPath, utilPath, repositoryPath, factoryPath, servicePath, controllerPath]
        },
        jasmine: {
            pivotal: {
                // Your project's source files
                src: [config, utilPath, repositoryPath, factoryPath, servicePath, controllerPath],
                // Your Jasmine spec files
                options: {
                    vendor: ['bower_components/assets/dist/zmart.js','libs/**/*.js', "specs/libs/*.js"],
                    specs: specsPath,
                    // Your spec helper files
                    helpers: helperPath
                }
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/dist/zmart.min.js': ['libs/**/*.js', config, utilPath, repositoryPath, factoryPath, servicePath, controllerPath].concat(orderFiles)
                }
            }
        },
        zhIdGenerator: {
             options: {
                jsonPath: ["../mobile/assets/json"],
                menuPath: ["./assets/json/menu.json"],
                replaceIds: false
            }
        },
        watch: {
            pivotal: {
                files: [config, utilPath, repositoryPath, factoryPath, servicePath, controllerPath, specsPath],
                tasks: ['jshint', 'uglify', 'concat']
            }
        },
        buildEnvironment: {
            development: {
                configFile: "../config/environment.json"
            },
            test: {
                configFile: "../config/environmentTest.json"
            },
            deploy: {
                configFile: "../config/environmentDeploy.json"
            },
            support: {
                configFile: "../config/environmentSupport.json"
            },
            srcFolders: ['../mobile', '../mobile/config', '../mobile/assets/js', '../mobile/assets/json', '../backend', '../backend/config']
        },
        buildAclStruct: {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        zhWords: {
            options: {
                serverUrl: "https://rc-translate.teknisa.com/backend/index.php",
                wordsPath: "./assets/json/words/",
                organization: "20",
                project: "25918"
            }
        }
    });

    function getLibPaths() {
        return libFiles.map(function(fileName) {
            return 'bower_components/' + fileName;
        });
    }

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/style.css');
        paths.push('./assets/css/zmart-periods-list.css');
        paths.push('./assets/css/zmart-compare-periods.css');
        paths.push('./assets/css/dashboard-compare-periods.css');
        paths.push('bower_components/zh-customizations-service/src/style/style.css');
        paths.push('bower_components/zh-crosstab/dist/zh-crosstab.css');
        paths.push('bower_components/zh-charts/dist/zh-charts.css');
        paths.push('bower_components/zh-select-autocomplete/dist/style/colors/styles-zeedhi.css');
        paths.push('bower_components/zh-grid-navigation/src/style.css');
        paths.push('!./assets/css/app.css');
        return paths;
    }

    grunt.loadNpmTasks('grunt-bump');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-jasmine');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-mkdir');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-words');

    grunt.registerTask('default', ['jshint', 'uglify', 'concat', 'mkdir']);
    grunt.registerTask('commit', ['jshint', 'uglify', 'concat']);
    grunt.registerTask('development', ['jshint', 'uglify', 'concat', 'buildEnvironment:development']);
    grunt.registerTask('support', ['jshint', 'uglify', 'concat', 'buildEnvironment:support']);
    grunt.registerTask('test', ['jshint', 'uglify', 'concat', 'buildEnvironment:test']);
    grunt.registerTask('deploy', ['buildEnvironment:deploy']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);
};
