(function() {
	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	ZeedhiDirectives.directive(
		'zhFloatingControl', ["templateManager", "ScreenService", "$timeout", "eventEngine", "ApplicationContext", "$window", "$rootScope", "ZhUtil", "$compile", zhFloatingControlDirective]
	);

	function zhFloatingControlDirective(templateManager, ScreenService, $timeout, eventEngine, ApplicationContext, $window, $rootScope, zhUtil, $compile) {
		return {
			priority: 1,
			link: function($scope, element, attrs, controller) {
				var component = $scope.widget;
				if (component.floatingControl === false) return;

				var defaultOptions = {
					position: "bottom right",
					draggable: true,
					refreshAction: true,
					searchAction: true,
					configAction: false,
					actions: [],
					statuses: []
				};
				component.floatingControl = Util.extend(defaultOptions, component.floatingControl || {});

				component.floatingControl.addStatus = function(status) {
					component.floatingControl.statuses.push(status);
					component.floatingControl.statuses.sort(function(a, b) {
						return a.priority - b.priority;
					});
				};
				component.floatingControl.statuses = [];

				component.floatingControl.addStatus({
					priority: 10,
					classString: 'checking',
					icon: 'checked',
					conditionFunction: function() {
						return component.showCheckbox && component.getCheckedRows && component.getCheckedRows().length > 0;
					},
					infoTemplate: '<span data-zh-icon="close-x no-border icon-white" data-zh-touchstart="clearInfo()"></span><span class="text" data-ng-bind-html="infoText"></span>',
					scope: $scope
				});

				$scope.getFloatingStatus = function() {
					var result;
					component.floatingControl.statuses.some(function(status) {
						if (status.conditionFunction()) {
							result = status;
							return true;
						}
					});

					return result;
				};

				var lastTemplate;
				$scope.getFloatingStatusClass = function() {
					var status, statusClass;
					var infoElement = element.find('.control-info');
					element.removeClass('has-status');
					status = $scope.getFloatingStatus();
					if (status) {
						if(status.classString){
							statusClass = status.classString;
							element.addClass('has-status');
						}

						if (status.infoTemplate) {
							var template;
							if (Util.isFunction(status.infoTemplate)) {
								template = status.infoTemplate();
							} else {
								template = status.infoTemplate;
							}
							if (lastTemplate != template) {
								lastTemplate = template;
								var info = $compile(template)(status.scope);
								infoElement.empty();
								infoElement.append(info);
							}
						}
					} else {
						infoElement.empty();
						lastTemplate = "";
					}

					return statusClass;
				};

				$scope.getFloatingStatusIcon = function() {
					var status, statusIcon;
					status = $scope.getFloatingStatus();
					if (status && status.icon) {
						statusIcon = status.icon;
					}
					return statusIcon;
				};

				var container = element.parent().parent();
				var isPopup = container.parents('.popup-container').length > 0;

				// default positioning
				var position = component.floatingControl.position || 'right bottom';
				if (position.indexOf('top') !== -1) {
					element.addClass('pos-top');
				}
				if (position.indexOf('bottom') !== -1) {
					element.addClass('pos-bottom');
				}
				if (position.indexOf('left') !== -1) {
					element.addClass('pos-left');
				}
				if (position.indexOf('right') !== -1) {
					element.addClass('pos-right');
				}

				function snapPosition() {
					element.animate({
						top: element.hasClass('pos-top') ? 15 : container.height() - element.height() - 15,
						left: element.hasClass('pos-left') ? 15 : container.width() - element.width() + 10
					}, 200, function() {
						element.css({
							top: element.hasClass('pos-top') ? 15 : 'auto',
							bottom: element.hasClass('pos-bottom') ? 15 : 'auto',
							left: element.hasClass('pos-left') ? 15 : 'auto',
							right: element.hasClass('pos-right') ? 15 : 'auto'
						});
					}).animate({
						opacity: 1
					}, 200);
				}
				$timeout(snapPosition, 100);
				$scope._closeTimeout = $timeout(closeActionMenu, 3000);

				// draggable
				if (component.floatingControl.draggable) {
					element.draggable({
						containment: container,
						opacity: 0.5,
						delay: 100,
						drag: function(event, ui) {
							closeOpenedActions();
							ui.position.left = Math.min(Math.max(15, ui.position.left), container.width() - element.width() + 10);
							ui.position.top = Math.min(Math.max(15, ui.position.top), container.height() - element.height() - 15);

							if (ui.position.left < container.width() / 2) {
								element.removeClass('pos-right');
								element.addClass('pos-left');
							} else {
								element.addClass('pos-right');
								element.removeClass('pos-left');
							}
							if (ui.position.top < container.height() / 2) {
								element.removeClass('pos-bottom');
								element.addClass('pos-top');
							} else {
								element.addClass('pos-bottom');
								element.removeClass('pos-top');
							}
						},
						stop: function(event, ui) {
							snapPosition();
							$(event.toElement).one('click tap', function(e) {
								e.stopImmediatePropagation();
							});
						}
					});
				}

				// checking rows
				component.updateCheckingState = function(text, updateTemplate) {
					$scope.infoText = text;

					if (Util.isUndefined(updateTemplate) || updateTemplate) {
						templateManager.updateTemplate();
					}
				};

				$timeout(function() {
					if (templateManager.container) {
						templateManager.container.updateCheckedRows.apply(templateManager.container);
					}
				});

				function closeOpenedActions() {
					var actions = element.find('.float-action.opened');
					for (var i = 0; i < actions.length; i++) {
						actions.eq(i).removeClass('opened');
					}
				}
				
				$scope.closeOpenedActions = closeOpenedActions;
				element[0].closeActionMenu = closeActionMenu;
				
				$scope.clearInfo = function() {
					$scope.clearCheckedRows();
				};

				$scope.clearCheckedRows = function() {
					if (component.hasCheckedRows()) {
						component.unSelectAll();
					}
				};

				function closeActionMenu() {
					component.updateCheckingState($scope.infoText);
					closeOpenedActions();
					element.removeClass('opened');
				}
				component.closeActionMenu = closeActionMenu;

				$scope.toggleActionMenu = Util.buildDebounceMethod(function(e) {
					$timeout.cancel($scope._closeTimeout);
					if (element.hasClass('opened')) {
						$scope.$broadcast('floating-control-before-close');
						closeActionMenu();
					} else {
						element.addClass('opened');
						$scope.$broadcast('floating-control-after-close');
					}
				}, 100, true);

				$scope.toggleAction = function(e) {
					$timeout.cancel($scope._closeTimeout);
					var action = $(e.target).closest('.float-action');
					if (action.hasClass('opened')) {
						action.removeClass('opened');
					} else {
						closeOpenedActions();
						action.addClass('opened');
						action.find('input').focus().select();
					}
				};

				if (!window._clickRegistered) {
					window._clickRegistered = true;

					$(window).on('click.floatingControl', function(e) {
						if ($(e.target).parents('.zh-floating-control').length === 0) {
							if (ScreenService.isPopupOpen()) {
								$('.popup-container .zh-floating-control').each(function(index, el) {
									if(el.closeActionMenu) {
										el.closeActionMenu();
									}
								});
							}
							else {
								$('.zh-floating-control').each(function(index, el) {
									if(el.closeActionMenu) {
										el.closeActionMenu();
									}
								});
							}
						}
					});
				}
				
				$scope.refreshWidget = function() {
					$timeout.cancel($scope._closeTimeout);
					closeOpenedActions();
					return component.reload();
				};
				component.refreshWidget = $scope.refreshWidget;

				if (component.floatingControl.configAction) {
					ScreenService.startEditWidget(component);
				}

				$scope.ControlInfoIconClick = function(data) {
					// to do: change this to a more generic function
					$scope.$broadcast('control-info-icon-click', data);
				};

				if (!templateManager.project._unWatchFloatingActions) {
					templateManager.project._unWatchFloatingActions = $rootScope.$watchCollection(function () {
						return templateManager.project.customFloatingActions;
					}, function(newValue) {
						if (newValue) {
							templateManager.project._orderedCustomFloatingActions = newValue.sort(function(a, b) {
								return (a.order || 0) - (b.order || 0);
							});
						}
					});
				}

				$scope.getCustomActions = function() {
					return templateManager.project._orderedCustomFloatingActions;
				};
			}
		};
	}
})();