function IntervalRegister(DecodeValueFieldService, validateEngine, ScreenService) {
    var self = this;

    /**
     * DecodeValueField Register
     **/
    var DEFAULT_DISPLAY_FORMAT = 'YYYY-MM-DD';
    var DEFAULT_DATE_FORMAT = 'YYYY-MM-DD';

    self.decodeValueField = function(field, fieldRow) {
        var displayValue = "";
        var dateFormat = field.dateFormat || DEFAULT_DATE_FORMAT;
        var formattedDate = field.displayFormat || DEFAULT_DISPLAY_FORMAT;
        if(fieldRow){
            if (fieldRow.end || fieldRow.start || fieldRow.period) {
                var beginDate = fieldRow.start ? moment(fieldRow.start, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                var endDate = fieldRow.end ? moment(fieldRow.end, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                if (!fieldRow.period) {
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate;
                    return displayValue;
                } else {
                    var periodLabel = fieldRow.period.label;
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate + ' (' + periodLabel + ') ';
                    return displayValue;
                }

            }
        }

        return ScreenService.i18n("From ") + formattedDate.replace(/[a-z]/gi, '_') + ScreenService.i18n(" to ") + formattedDate.replace(/[a-z]/gi, '_');
    };

    DecodeValueFieldService.register("interval.html", self.decodeValueField);

    /**
     * ValidateEngine Register
     **/
    function required(value, validationAttrs) {
        return {
            "valid": requiredStart(value, validationAttrs).valid && requiredEnd(value, validationAttrs).valid,
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Required field")
        };
    }

    function requiredStart(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'start');   
    }

    function requiredEnd(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'end');
    }

    function buildRequiredMessage(value, validationAttrs, property) {
        return {
            "valid": value && Util.isObject(value) && !Util.isEmptyOrBlank(value[property]),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n('Required field')
        };
    }

    var registeredValidations = validateEngine.getMappedValidations();
    var validationsToRegister = {
        "requiredInterval": required,
        "requiredIntervalStart": requiredStart,
        "requiredIntervalEnd": requiredEnd
    };

    for (var validationName in validationsToRegister) {
        if (!registeredValidations.hasOwnProperty(validationName)) {
            validateEngine.registerValidation(validationName, validationsToRegister[validationName]);
        }
    }
}

Configuration(IntervalRegister);