function CarouselWidgetCreator(metaDataFactory, ScreenService) {

	this.createCaroulselWidget = function(gridWidget, functionsToInsertOnWidget){
		var caroulsel = {
			"id": 1,
			"container": gridWidget.container,
			"name": "carouselWidget",
			"showAsModal": true,
			"isVisible": true,
			"template": "widget/carousel.html",
			"actions": []
		};
		var caroulselFactored = metaDataFactory.widgetFactory(caroulsel);
		caroulselFactored.widgets = createChildrenCaroulselWidget(gridWidget, caroulselFactored, functionsToInsertOnWidget);
		return caroulselFactored;
	};

	var createChildrenCaroulselWidget = function(gridWidget, caroulselWidget, functionsToInsertOnWidget){
		var children = [
			createPreferencesWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsPreferencesListWidget),
			createPreferenceDetailsWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsPreferenceFieldsListWidget),
			createPropertyPreferencesWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsFieldsPropertiesListWidget),
			createNewPreferenceWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsNewPreferenceWidget),
			createChangePreferenceTypeWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsChangePreferenceTypeWidget)
		];
		return children;
	};

	function createPreferencesWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "popUpPreferencesList",
			"label": ScreenService.i18n("Layouts"),
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsPreferencesWidget,
			"floatingControl": {
				"refreshAction": false,
				"customizationAction": {
					"layout": false,
					"filter": false,
					"view": false
				},
				"actions": []
			},
			"events": [
				{
					"id": 2,
					"name": "WidgetOnEnter",
					"code": "args.owner.preferencesListOnEnter(args.owner);"
				},
				{
					"id": 3,
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.openEditPreference(args.owner, args.row);"
				}
			],
			"parentWidget": parentWidget,
			"dataSource": {
				"data": [],
				"localStorage": false,
				"rest": false
			},
			"fields": [
				{
					"id": 1,
					"name": "label",
					"label": ScreenService.i18n("Layout name"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 1,
					"name": "preferenceTypeLabel",
					"label": ScreenService.i18n("Type"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"name": "pdfReportOrientation",
					"label": ScreenService.i18n("PDF Report Orientation"),
					"isVisible": true,
					"showOnList": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": true,
					"dataSource": {
						"localStorage": false,
						"data":[
							{
								"value": "PORTRAIT",
								"label": "Portrait"
							},
							{
								"value": "LANDSCAPE",
								"label": "Landscape"
							}
						]
					}
				},
				{
					"id": 4,
					"name": "defaultPreference",
					"isVisible": false,
					"showOnList": true,
					"label": ScreenService.i18n("Default"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"hideCheckAllButton": true,
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.setAsDefaultPreference(args.row, args.owner.field.widget);"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createPreferenceDetailsWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 2,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "PreferenceDetailsWidget",
			"showDescriptionOrder": false,
			"label": "teste",
			"labelAction": true,
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsPreferenceDetailsWidget,
			"events": eventsPreferenceDetailWidget,
			"styles": [
				{
					"cssClass" : "last-field",
					"condition" : "row.sequenceInList == row.qtdRows"
				},
				{
					"cssClass" : "first-field",
					"condition" : "row.sequenceInList == 1"
				}
			],
			"parentWidget": parentWidget,
			"dataSource": {
				"data": []
			},
			"fields": [
				{
					"id": 1,
					"label": ScreenService.i18n("Column name"),
					"name": "label",
					"showOnList": true
				},
				{
					"id": 4,
					"name": "visibility",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Column visibility"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.changePreferenceVisibility(args.row, args.owner.field.widget);"
					}]
				},
				{
					"id": 4,
					"name": "fixed",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Fix column"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": []
				},
				{
					"id": 4,
					"name": "grouped",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Group by column"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": []
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createPropertyPreferencesWidget(gridWidget, parentWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": 9999,
			"name": "properties_widget",
			"showAsModal": true,
			"container": gridWidget.container,
			"parentWidget": parentWidget,
			"label": ScreenService.i18n("Column properties") + " - ",
			"showDescriptionOrder": false,
			"template": "widget/form.html",
			"isVisible": true,
			"popupNoBlock": false,
			"fields": propertiesField,
			"dataSource": [],
			"dataSourceFilter": [],
			"events": [],
			"actions": actionsPropertiesWidget
		});
		return Util.extend(widget, functionsToInject);
	}

	function createNewPreferenceWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"name": "newPreferenceWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Create a new layout"),
			"template": "widget/form.html",
			"parentWidget": parentWidget,
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "name",
					"label": ScreenService.i18n("Name"),
					"class": "6 col-md-6",
					"isVisible": true,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/text-edit.html",
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "type",
					"label": ScreenService.i18n("Type"),
					"class": "3 col-md-3",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "pdfReportOrientation",
					"label": ScreenService.i18n("PDF Report Orientation"),
					"class": "3 col-md-3",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": true,
					"dataSource": {
						"localStorage": false,
						"data":[
							{
								"value": "PORTRAIT",
								"label": "Portrait"
							},
							{
								"value": "LANDSCAPE",
								"label": "Landscape"
							}
						]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.closePopupAndRunDigest()"
					}]
				},
				{
					"id": 1,
					"name": "addPreference",
					"label": ScreenService.i18n("Continue"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.createNewPreference(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createChangePreferenceTypeWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 8998,
			"name": "changePreferenceTypeWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Change layout type"),
			"template": "widget/form.html",
			"parentWidget": parentWidget,
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "preferenceType",
					"label": ScreenService.i18n("New layout type"),
					"class": "12",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.closePopupAndRunDigest()"
					}]
				},
				{
					"id": 1,
					"name": "addPreference",
					"label": ScreenService.i18n("Save"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.saveTransferLayoutClick(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	var actionsPreferencesWidget = [
		{
			"id": 2,
			"name": "NEW",
			"isVisible": true,
			"showAsAction": "always",
			"icon": "plus icon-white",
			"tooltip": ScreenService.i18n("New"),
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.openNewPreferenceWidget(args.owner.widget)"
			}]
		},
		{
			"id": 3,
			"name": "cancelAction",
			"label": ScreenService.i18n("Close"),
			"showAsAction": "back",
			"hideIcon": true,
			"isVisible": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.closePopupAndRunDigest();"
			}]
		},
		{
			"id": 4,
			"name": "deletePreference",
			"isVisible": true,
			"icon": "trash",
			"showAsAction": "checked_rows",
			"condition": "row.acl_edit",
			"tooltip": ScreenService.i18n("Delete preference"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.deletePreferenceClick(args.owner.widget);"
				}
			]
		}
	];

	var actionsPropertiesWidget = [
		{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.saveFieldChanges(args.owner.widget, args.row)"
			}]
		}
	];

	var actionsPreferenceDetailsWidget = [
		{
			"id": 0,
			"name": "saveAndApply",
			"isVisible": true,
			"showAsAction": "never",
			"label": ScreenService.i18n("Save & Apply"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.saveAndApplyPreference(args.owner.widget)"
				}
			]
		},
		{
			"id": 2,
			"name": "backAction",
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
				}
			]
		},
		{
			"id": 3,
			"name": "moveUpField",
			"isVisible": true,
			"icon": "up",
			"showAsAction":"checked_rows",
			"tooltip": ScreenService.i18n("Move field up"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.changeOrder(-1, args.owner.widget);"
				}
			]
		},
		{
			"id": 4,
			"name": "moveDownField",
			"isVisible": true,
			"icon": "down",
			"showAsAction": "checked_rows",
			"tooltip": ScreenService.i18n("Move field down"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.changeOrder(1, args.owner.widget);"
				}
			]
		}
	];

	var eventsPreferenceDetailWidget = [
		{
			"id": 1,
			"name": "WidgetAfterMoveRow",
			"code": "args.owner.openPopupPreferencesProperty(args.owner, args.row)"
		}
	];

	var propertiesField = [
		{
			"name": "label",
			"label": ScreenService.i18n("Label"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/text-edit.html",
			"events": []
		},
		{
			"name": "align",
			"label": ScreenService.i18n("Alignment"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/select-native.html",
			"valueField": "value",
			"descriptionField": "value",
			"events": [],
			"dataSource": {
				"data":[
					{ "value": "left" },
					{ "value": "center" },
					{ "value": "right" }
				]
			}
		},
		{
			"name": "expression",
			"label": ScreenService.i18n("Group expression"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/select-native.html",
			"valueField": "value",
			"descriptionField": "value",
			"defaultOption": "left",
			"events": [],
			"dataSource": {
				"data":[
					{ "value": "COUNT" },
					{ "value": "SUM" },
					{ "value": "AVG" }
				]
			}
		},
		{
			"name": "minWidth",
			"label": ScreenService.i18n("Minimun width (px)"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/number.html",
			"spin": true,
			"minValue": 0,
			"events": []
		},
		{
			"name": "maxWidth",
			"label": ScreenService.i18n("Maximum width (px)"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/number.html",
			"minValue": 0,
			"spin": true,
			"events": []
		},
		{
			"name": "reportColumnSize",
			"label": ScreenService.i18n("PDF Report Column Size"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/text.html",
			"events": []
		}
	];

}

Configuration(function(ContextRegister) {
	ContextRegister.register("carouselWidgetCreator", CarouselWidgetCreator);
});
