<?php
namespace Teknisa\Model\Teknisa\Repositories;

class OmConfigMedidaVariavel extends \Doctrine\ORM\EntityRepository {

    public function findVariableMeasurementSettings($conditions, $nrProdutoId){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT CMV.ID, CMV.IDATIVO, CMV.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV,
		                CMV.PHP_SCRIPT_COUNT, CMV.PHP_SCRIPT_LOG, CMV.NRPRODUTO_ID,
		                CMV.QTD_DIAS_AVISO, CMV.QTD_DIAS_REAPLICAR
                   FROM OM_CONFIG_MEDIDA_VARIAVEL CMV
                   JOIN FCT_TIPOMEDIDASERV TMS ON TMS.CDTPMEDIDASERV = CMV.CDTPMEDIDASERV
                  WHERE NRPRODUTO_ID = :NRPRODUTO_ID )
		 WHERE 1=1
		      $conditions
SQL;
        $param = array( 'NRPRODUTO_ID' => $nrProdutoId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findTipoMedidaServ($conditions, $nrOrg, $limit, $offset) {
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT CDTPMEDIDASERV, DSTPMEDIDASERV, IDTPCOLETAMEDIDASERV, IDATIVO
                   FROM FCT_TIPOMEDIDASERV
                  WHERE NRORG = :NRORG )
		 WHERE 1 = 1
		       $conditions
      ORDER BY DSTPMEDIDASERV
SQL;
        $param = array( 'NRORG' => $nrOrg );
        $query = $this->getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findVariableMeasureByExternalProductId($productId){
        $query = <<<SQL
		SELECT CMV.ID, CMV.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV, CMV.PHP_SCRIPT_COUNT,
               CMV.PHP_SCRIPT_LOG, CMV.QTD_DIAS_AVISO, CMV.QTD_DIAS_REAPLICAR,
               CMV.NRPRODUTO_ID, PRO.NMPRODUTO, PRO.NREXTERNAL_PRODUCT_ID
          FROM OM_CONFIG_MEDIDA_VARIAVEL CMV
          JOIN OM_PRODUTO PRO ON PRO.ID = CMV.NRPRODUTO_ID
          JOIN FCT_TIPOMEDIDASERV TMS ON TMS.CDTPMEDIDASERV = CMV.CDTPMEDIDASERV
         WHERE PRO.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
           AND CMV.IDATIVO = 'S'
SQL;
        $param = array( 'PRODUCT_ID' => $productId );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findActualCustomerContracts($collectType = \Teknisa\Model\Teknisa\Entities\OmConfigMedidaVariavel::VARIABLE_MEASURE_COLLECT_TEKNISA_TYPE,
                                                $productId = null, $organizationId = null, $variableMeasureId = null) {
        $productFilter = $variableMeasureFilter = $organizationFilter = '';
        if($productId) {
            $productFilter = "AND PRO.NREXTERNAL_PRODUCT_ID = $productId";
        }
        if($variableMeasureId) {
            $variableMeasureFilter = "AND REL.CDTPMEDIDAVARIAVEL = '$variableMeasureId'";
        }
        if($organizationId) {
            $organizationFilter = "AND ORG.NRORG = $organizationId";
        }
        $query = <<<SQL
		SELECT ORC.DSREFERENCIAORCA, VENC.NRSEQCONTRATO,
               VENC.NRMEDICAOFATURA AS NRVENCPREVCONT,
               CLI.NMRAZSOCCLIE, CLI.NMFANTCLIE, ORG.NRORG,
               TO_CHAR(VENC.DTINICOMPET, 'DD/MM/YYYY') AS DTINICOMPET,
               TO_CHAR(VENC.DTFINCOMPET, 'DD/MM/YYYY') AS DTFINCOMPET,
               TMS.CDTPMEDIDASERV, TMS.DSTPMEDIDASERV,
               PRO.NREXTERNAL_PRODUCT_ID, PRO.NMPRODUTO
          FROM FCT_MEDICAOFATURA VENC
    INNER JOIN FCT_CONTRATO CONT
            ON VENC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
           AND VENC.NRORG         = CONT.NRORG
    INNER JOIN FCT_REFEORCACONT REF
            ON CONT.NRSEQCONTRATO = REF.NRSEQCONTRATO
           AND CONT.NRORG         = REF.NRORG
    INNER JOIN FCT_ORCAMENTO ORC
            ON REF.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
           AND REF.NRORG          = ORC.NRORG
    INNER JOIN ORGANIZACAO ORG
            ON CONT.NRPARCNEGOCIO = ORG.NRPARCNEGOCIO
               $organizationFilter
    INNER JOIN CLIENTE CLI
            ON ORG.NRPARCNEGOCIO  = CLI.NRPARCNEGOCIO
    INNER JOIN FCT_RELACMEDIDAPROPOSTA REL
            ON REL.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
           AND REL.NRORG          = ORC.NRORG
    INNER JOIN OM_PRODUTO PRO
            ON TO_CHAR(PRO.ID) = REPLACE(REL.IDELEMENTORAIZ, 'P_', '')
    INNER JOIN FCT_TIPOMEDIDASERV TMS
            ON TMS.CDTPMEDIDASERV = REL.CDTPMEDIDAVARIAVEL
     LEFT JOIN FCT_FATURACONT FNT
            ON FNT.NRMEDICAOFATURA = VENC.NRMEDICAOFATURA
         WHERE CONT.IDSITUACAOCONTRATO = 'A'
           AND TRUNC(SYSDATE) BETWEEN TRUNC(VENC.DTINICOMPET) AND TRUNC(VENC.DTFINCOMPET)
           AND TMS.IDTPCOLETAMEDIDASERV = :IDTPCOLETAMEDIDASERV
           AND FNT.NRMEDICAOFATURA IS NULL
               $productFilter
               $variableMeasureFilter
      ORDER BY CLI.NMFANTCLIE, PRO.NMPRODUTO
SQL;
        $param = array (
            'IDTPCOLETAMEDIDASERV' => $collectType
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    private function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $max = $offset + $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }

        return $query;
    }
}