"use strict";

module.exports = function (grunt) {
    var menuJsonFile = "/menu.json";
    var containersFile = "/containers.json";
    var aclStructJsonFile = "/aclStruct.json";
    var productBaseName = "product";
    var packageMap = {};

    function readJSON(filePath) {
        filePath = "./" + filePath;
        return grunt.file.readJSON(filePath, { encoding: "utf8" });
    }

    function updatePackageMap(){
        Object.keys(packageMap).forEach(function(key){
            var path = packageMap[key];
            packageMap[key] = path.replace(menuJsonFile, "").replace(aclStructJsonFile, "");
        });
    }

    function getContainerPath(containerPackage, containerName){
        var containerPath = packageMap[containerPackage];
        if(containerPath){
            containerPath += "/containers/" + containerName + ".json";
        } else {
            grunt.log.warn("Package " + containerPackage + " not found in menuConfig.");
        }
        return containerPath;
    }

    function getContainer(windowName){
        var nameParts = windowName.split("#");
        var containerPackage = nameParts[0];
        var containerName = nameParts[1];

        if(!containerName){
            containerName = containerPackage;
            containerPackage = productBaseName;
        }

        var containerPath = getContainerPath(containerPackage, containerName);
        var container = containerPath ? readJSON(containerPath) : {};
        return container;
    }

    function addContainerId(menus){
        menus.forEach(function(menu){
            if(menu.windowName){
                var container = getContainer(menu.windowName);
                menu.containerId = container.id;
                menu.containerName = container.name;
                menu.containerLabel = container.label || container.name;
            } else if(menu.menus){
                addContainerId(menu.menus);
            }
        });
        return menus;
    }

    function readContainerMenu(productJsonPath){
        return readJSON(productJsonPath).zeedhi_project.groupMenu;
    }

    function buildProductAclStruct(productJsonPath){
        var completeMenu;
        try {
            completeMenu = readJSON(productJsonPath + menuJsonFile);
        } catch(err){
            if(err.origError.code === "ENOENT"){
                completeMenu = readContainerMenu(productJsonPath + containersFile);
            } else {
                grunt.fail.fatal(err);
            }
        }
        return addContainerId(completeMenu);
    }

    function saveAclStructFile(){
        var productPath = packageMap[productBaseName];
        var aclStruct = buildProductAclStruct(productPath);
        var content = JSON.stringify(aclStruct, null, 4);
        var path = "./" + productPath + aclStructJsonFile;
        grunt.file.write(path, content, { encoding: 'utf8' });
    }

    grunt.registerTask("buildAclStruct", "Generate aclStruct.json for menu/container items", function(){
        var options = this.options({modulesMenuFile: "./menuConfig.json"});
        var configs = readJSON(options.modulesMenuFile);
        var configProductName = configs.productBaseName;
        if(configProductName){
            productBaseName = configProductName;
        }

        packageMap = configs.modulesMenuFile || {};

        if(!packageMap[productBaseName]){
            packageMap[productBaseName] = configs.productMenuFile;
        }

        updatePackageMap();
        saveAclStructFile();
    });
};
