<?php
namespace Teknisa\Model\Teknisa\Repositories;


class OmProdutoOrgVersao extends \Doctrine\ORM\EntityRepository {

    public function findProductAvailableVersions($conditions, $nrProdutoOrgId){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT OV.ID, OV.NRPRODUTO_ORG_ID, OV.IDATIVO, PV.ID AS PRODUCT_VERSION_ID,
                        PV.VERSION || '.' || PV.REVISION AS VERSION, PV.IS_LTS
                   FROM OM_PRODUTO_ORG_VERSAO OV
                   JOIN PRODUCT_VERSION PV ON PV.ID = OV.PRODUCT_VERSION_ID
                  WHERE OV.NRPRODUTO_ORG_ID = :NRPRODUTO_ORG_ID
                  ORDER BY PV.VERSION DESC, PV.REVISION DESC)
		 WHERE 1 = 1
		      $conditions
SQL;
        $param = array( 'NRPRODUTO_ORG_ID' => $nrProdutoOrgId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['ID'] = intval($register['ID']);
        }
        return $registers;
    }

    public function findLTSVersions($conditions, $nrProdutoId, $limit, $offset, $notInsertedOnly, $productOrgId = null) {
        $notInsertedFilter = '';
        if($notInsertedOnly) {
            $notInsertedFilter = 'AND NOT EXISTS (SELECT OV.ID
                                                    FROM OM_PRODUTO_ORG_VERSAO OV
                                                   WHERE OV.PRODUCT_VERSION_ID = PV.ID
                                                     AND OV.NRPRODUTO_ORG_ID = :NRPRODUTO_ORG_ID)';
        }
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT PV.ID AS PRODUCT_VERSION_ID, PV.VERSION || '.' || PV.REVISION AS VERSION, PV.IS_LTS
                   FROM PRODUCT_VERSION PV
                   JOIN OM_PRODUTO OP ON PV.PRODUCT_ID = OP.NREXTERNAL_PRODUCT_ID
                  WHERE OP.ID = :PRODUCT_ID
                    AND PV.IS_REVISION = 'Y'
                        $notInsertedFilter
               ORDER BY PV.VERSION DESC, PV.REVISION DESC)
		 WHERE 1 = 1
		      $conditions
SQL;
        $param = array( 'PRODUCT_ID' => $nrProdutoId );
        if($productOrgId) {
            $param['NRPRODUTO_ORG_ID'] = $productOrgId;
        }
        $query = $this->getPaginatedQuery($query, $limit, $offset);
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        foreach($registers as &$register){
            $register['PRODUCT_VERSION_ID'] = intval($register['PRODUCT_VERSION_ID']);
        }
        return $registers;
    }

    private function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $max = $offset + $limit;
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $max . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $max;
            }
        }

        return $query;
    }

    public function findVersionModulesConfiguration($conditions, $productVersionId){
        $query = <<<SQL
		SELECT *
		  FROM ( SELECT MODULES_CONFIGURATION
                   FROM PRODUCT_VERSION
                  WHERE ID = :PRODUCT_VERSION_ID )
		 WHERE 1 = 1
		      $conditions
SQL;
        $param = array( 'PRODUCT_VERSION_ID' => $productVersionId );
        $registers = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return empty($registers) ? array() : $registers[0];
    }

    public function findAvailableVersionsByProductAndCustomer($productId, $customerCode){
        $query = <<<SQL
		SELECT PRODUCT_VERSION_ID, VERSION || '.' || REVISION AS VERSION
		  FROM ( SELECT PV.PRODUCT_VERSION_ID, PN.VERSION, PN.REVISION
                   FROM OM_PRODUTO_ORG_VERSAO PV
                   JOIN OM_PRODUTO_ORG PO ON PO.ID = PV.NRPRODUTO_ORG_ID
                   JOIN OM_PRODUTO PR ON PR.ID = PO.NRPRODUTO_ID
                   JOIN PRODUCT_VERSION PN ON PN.ID = PV.PRODUCT_VERSION_ID
                   JOIN ORGANIZACAO OG ON OG.NRORG = PO.NRORG_ID
                   JOIN CLIENTE CL ON CL.NRPARCNEGOCIO = OG.NRPARCNEGOCIO
                  WHERE PR.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
                    AND PV.IDATIVO = 'S'
                    AND CL.CDCLIENTE = :CDCLIENTE
                  UNION
                 SELECT PRODUCT_VERSION_ID, VERSION, REVISION
                   FROM (SELECT PN.ID AS PRODUCT_VERSION_ID, PN.VERSION, PN.REVISION
                           FROM PRODUCT_VERSION PN
                           JOIN PRODUCT PR ON PR.ID = PN.PRODUCT_ID
                          WHERE PR.ID = :PRODUCT_ID
                            AND PN.IS_REVISION = 'Y'
                       ORDER BY PN.VERSION DESC, PN.REVISION DESC)
                  WHERE ROWNUM = 1)
      ORDER BY VERSION DESC, REVISION DESC
SQL;
        $param = array( 'PRODUCT_ID' => $productId, 'CDCLIENTE' => $customerCode );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function findRelatedCustomerVersionsByProduct($productId, $customerCode){
        $query = <<<SQL
		SELECT OV.ID, OV.PRODUCT_VERSION_ID, OV.IDATIVO, OO.ID AS OM_PRODUTO_ORG_ID
          FROM CLIENTE CL
          JOIN ORGANIZACAO OG           ON OG.NRPARCNEGOCIO = CL.NRPARCNEGOCIO
          JOIN OM_PRODUTO_ORG OO        ON OO.NRORG_ID = OG.NRORG
          JOIN OM_PRODUTO OP            ON OP.ID = OO.NRPRODUTO_ID
          JOIN OM_PRODUTO_ORG_VERSAO OV ON OV.NRPRODUTO_ORG_ID = OO.ID
         WHERE CL.CDCLIENTE = :CDCLIENTE
           AND OP.NREXTERNAL_PRODUCT_ID = :PRODUCT_ID
SQL;
        $param = array( 'PRODUCT_ID' => $productId, 'CDCLIENTE' => $customerCode );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

}