/*jshint esversion: 6 */
"use strict";

module.exports = function(grunt){
    var rp = require('request-promise');
    var path = require('path');
    var fs = require('fs');
    var getWordsRoute = "getWordsByProject";
    var serverUrl = "https://translate.teknisa.com/backend/index.php";
    var wordsPath = "./assets/json/words/";

    var logVerbose = function(message){
        grunt.log.writeln('- '+message);
    };

    function validateOptions(){
        logVerbose('Validating options');
        var optionsNames = ['serverUrl', 'wordsPath', 'organization', 'project'];
        optionsNames.every(name => {
            if(typeof name !== 'string'){
                grunt.fail.fatal("Invalid arguments: all parameters should be strings.");
            }
        });
        logVerbose('Options OK');
    }

    function getAuthenticationFlags(options){
        var flags = grunt.option.flags();
        var neededFlags = ['organization', 'project'];
        var authenticationFlags = {};
        flags.forEach(flag => {
            var [property, value] = flag.split("=");
            property = property.substr(2, property.length);
            authenticationFlags[property] = value;
        });

        neededFlags = neededFlags.map(flagName => {
            if(!authenticationFlags[flagName] && !options[flagName]){
                grunt.fail.fatal("Flag '"+ flagName +"'is required.");
            }
            return authenticationFlags[flagName];
        });
        return neededFlags;
    }

    function buildWordsFolder(){
        logVerbose('Creating folder');
        if(!fs.existsSync(wordsPath)){
            fs.mkdirSync(wordsPath);
        }
        logVerbose('Folder created');
    }

    function buildWordsFiles(body){
        buildWordsFolder();
        logVerbose('Building files');
        if(!Array.isArray(body)){
            grunt.fail.fatal(body);
        }

        var langs = body.reduce(function(actual, word){
            var wordISO = word.LANGUAGE_ISO.toLowerCase();
            actual[wordISO] = actual[wordISO] || [];
            actual[wordISO].push(word);
            return actual;
        }, {});

        Object.keys(langs).forEach(function(lang){
            var fileName = wordsPath+lang+'.json';
            var wordsFileLangs = {};
            langs[lang].forEach(function(word){
                wordsFileLangs[word.WORDS_VALUE] = word.VALUE;
            });
            logVerbose(' Building '+fileName);
            fs.writeFileSync(fileName, JSON.stringify(wordsFileLangs, null, 4));
            grunt.log.writeln('>>'.green+' File '+ fileName +' saved!');
        });
    }

    function runZhWords(organizationId, projectId, done){
        if(serverUrl[serverUrl.length - 1] !== '/'){
            getWordsRoute = '/' + getWordsRoute;
        }

        var requestOptions = {
            method: 'POST',
            uri: serverUrl + getWordsRoute,
            json: true,
            body: {
                row: {
                    organizationId: organizationId,
                    projectId: projectId
                },
                requestType: 'Row'
            }
        };
        logVerbose('Making request');
        rp(requestOptions).then(response => {
            logVerbose('Request OK');
            buildWordsFiles(response);
            done();
        }).catch(err => {
            console.log(err.response.body);
            logVerbose(err.response.body);
            grunt.warn("Error on find.");
            done();
        });
    }

    grunt.registerTask("zhWords", "Generate words files for translate projects", function(args1){
        if(args1 !== 'verbose'){
            logVerbose = function(){};
        }

        logVerbose('Get flags');

        var done = this.async();
        var [organization, project] = getAuthenticationFlags(this.options());
        var options = this.options({
            serverUrl: serverUrl,
            wordsPath: wordsPath,
            organization: organization,
            project: project
        });

        serverUrl = options.serverUrl;
        wordsPath = path.normalize(options.wordsPath + '/');

        validateOptions();
        runZhWords(options.organization, options.project, done);
    });
};