<?php

namespace Teknisa\ACL\Controllers;

use Teknisa\ACL\Services\PerfAcessParc as PerfAcessParcService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\ACL\Util\InstanceProvider;

class PerfAcessParc extends Crud{

    const IS_NEW = "__is_new";

    protected $dataSourceName = "perfAcessParcOperador";
    protected $dataSourceNameSave = "perfAcessParc";

    /** @var PerfAcessParcService */
    protected $perfAcessParcService;

    public function __construct(Manager $manager, PerfAcessParcService $perfAcessParcService) {
        $this->perfAcessParcService = $perfAcessParcService;
        parent::__construct($manager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $request->getFilterCriteria()->addCondition(PerfAcessParcService::NRORG, $this->perfAcessParcService->getNrOrgFilter());
        parent::find($request, $response);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as $row) {
            if(!$row['NRPARCNEGOCIO']) {
                $businessPartnership = InstanceProvider::getService()->getOperator()->setBusinessPartnership(
                                $row['NMOPERADOR'], $row['NRINSCRICAOPARC'], $row['IDATIVO'], $row['CDOPERINCLUSAO'],
                                $row['CDTIPOINSCRICAO'], $row['CDOPERADOR']);
                $row['NRPARCNEGOCIO'] = $businessPartnership['NRPARCNEGOCIO'];
            }
            if ($row[self::IS_NEW]) {
                $row[PerfAcessParcService::NRPERFACESSPARC] = $this->perfAcessParcService->getMaxNrPerfilAcesso();
            }            
            $this->perfAcessParcService->setDefaultValues($row);
        }
        $originalDataSourceName = $this->dataSourceName;
        $this->dataSourceName = $this->dataSourceNameSave;
        parent::save($request, $response);
        $this->dataSourceName = $originalDataSourceName;
    }

    public function findPerfAcessParcOperador(DTO\Request\Filter $request, DTO\Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $orderBy       = $request->getFilterCriteria()->getOrderBy();
        $data = InstanceProvider::getService()->getPerfAcessParc()->findPerfAcessParcOperador($conditions, $orderBy);
        $response->addDataSet(new DataSet('findPerfAcessParcOperador', $data));

    }

}